;****************************************************************************
; MODULO PARA MODO DE VIDEO 19
;
; PRUEBA 6 - Pega una pintura en la pantalla.
;****************************************************************************

;****************************************************************************
; Define los valores.
;****************************************************************************

  PUBLIC MMV19_INICIA_MODO
  PUBLIC MMV19_FINALIZA_MODO
  PUBLIC MMV19_PEGA_PUNTO
  PUBLIC MMV19_PEGA_LINEA_HORIZONTAL
  PUBLIC MMV19_PEGA_LINEA_VERTICAL
  PUBLIC MMV19_PEGA_PINTURA

  INTERRUPCION_VIDEO = 010H
  FUNCION_PONE_MODO  = 000H
  MODO               = 19

  SEGMENTO_BIOS = 00040H
  CAMPO_MODO    = 00049H

  SEGMENTO_VIDEO = 0A000H

  ANCHO_PANTALLA = 320

CAMINO SEGMENT PUBLIC
  ASSUME CS:CAMINO,DS:CAMINO,SS:CAMINO,ES:CAMINO

;****************************************************************************
; Guarda el modo de video y pone el 19. Este modo tiene un rea de 320 * 200,
; y 256 colores.
;
; Escribe:
;   VIEJO_MODO
;****************************************************************************

  LABEL MMV19_INICIA_MODO NEAR
  INICIA_MODO PROC NEAR
    PUSH BX
    PUSH ES
    PUSH AX

;****************************************************************************
; Obtiene el modo de video. Este valor se encuentra en el segmento de datos
; del BIOS.
    MOV BX,SEGMENTO_BIOS
    MOV ES,BX
    MOV BX,CAMPO_MODO
    MOV AL,ES:[BX]

;****************************************************************************
; Almacena su valor.
    MOV VIEJO_MODO,AL

;****************************************************************************
; Cambia al nuevo modo.
    MOV AH,FUNCION_PONE_MODO
    MOV AL,MODO
    INT INTERRUPCION_VIDEO

    POP AX
    POP ES
    POP BX
    RET
  INICIA_MODO ENDP

;****************************************************************************
; Pone el viejo modo de video.
;
; Lee:
;   VIEJO_MODO
;****************************************************************************

  LABEL MMV19_FINALIZA_MODO NEAR
  FINALIZA_MODO PROC NEAR
    PUSH AX

    MOV AH,FUNCION_PONE_MODO
    MOV AL,VIEJO_MODO
    INT INTERRUPCION_VIDEO

    POP AX
    RET
  FINALIZA_MODO ENDP

;****************************************************************************
; Pega un punto en la pantalla.
;
; Necesita:
;   AL = color del punto
;      = 0..255
;      = 0, negro en la paleta convencional
;      = 15, blanco en la paleta convencional
;   DI = posicin en la pantalla
;      = 0..63999
;      = 0, la esquina superior izquierda
;      = 63999, la esquina inferior derecha
;
; Escribe:
;   SEGMENTO_VIDEO:DI
;****************************************************************************

  LABEL MMV19_PEGA_PUNTO NEAR
  PONE_PUNTO PROC NEAR
    PUSH BX
    PUSH ES
    PUSH DI

    MOV BX,SEGMENTO_VIDEO
    MOV ES,BX
    STOSB

    POP DI
    POP ES
    POP BX
    RET
  PONE_PUNTO ENDP

;****************************************************************************
; Pega una lnea horizontal en la pantalla.
;
; Necesita:
;   AL = color del punto
;   DI = inicio de la lnea (posicin superior izquierda)
;   CX = cantidad de puntos en la lnea
;      = 1..64000
;
; Escribe:
;   SEGMENTO_VIDEO:DI
;****************************************************************************

  LABEL MMV19_PEGA_LINEA_HORIZONTAL NEAR
  PEGA_LINEA_HORIZONTAL PROC NEAR
    PUSH BX
    PUSH ES
    PUSH DI
    PUSH CX

    MOV BX,SEGMENTO_VIDEO
    MOV ES,BX
    REP STOSB

    POP CX
    POP DI
    POP ES
    POP BX
    RET
  PEGA_LINEA_HORIZONTAL ENDP

;****************************************************************************
; Pega una lnea vertical en la pantalla.
;
; Necesita:
;   AL = color del punto
;   DI = inicio de la lnea (posicin superior)
;   CX = cantidad de puntos en la lnea
;
; Escribe:
;   SEGMENTO_VIDEO:DI
;****************************************************************************

  LABEL MMV19_PEGA_LINEA_VERTICAL NEAR
  PEGA_LINEA_VERTICAL PROC NEAR
    PUSH BX
    PUSH ES
    PUSH DI
    PUSH CX

    MOV BX,SEGMENTO_VIDEO
    MOV ES,BX

    CICLO_1:

;****************************************************************************
; Pega un punto y salta a la siguiente lnea en la pantalla.
    STOSB
    ADD DI,ANCHO_PANTALLA-1
    LOOP CICLO_1

    POP CX
    POP DI
    POP ES
    POP BX
    RET
  PEGA_LINEA_VERTICAL ENDP

;****************************************************************************
; Pega una pintura en la pantalla.
;
; Necesita:
;   DS:SI = direccin de la pintura
;   DI    = inicio en pantalla (posicin superior)
;   CX    = cantidad de puntos a pegar
;
; Lee:
;   DS:SI
;
; Escribe:
;   SEGMENTO_VIDEO:DI
;****************************************************************************

  LABEL MMV19_PEGA_PINTURA NEAR
  PEGA_PINTURA PROC NEAR
    PUSH BX
    PUSH ES
    PUSH CX
    PUSH SI
    PUSH DI

;****************************************************************************
; Establece el destino.
    MOV BX,SEGMENTO_VIDEO
    MOV ES,BX

;****************************************************************************
; Calcula la cantidad de words y bytes a pegar.
    MOV BX,CX
    AND BX,1
    SHR CX,1

;****************************************************************************
; Pega los words.
    REP MOVSW

;****************************************************************************
; Pega los bytes restantes.
    MOV CX,BX
    REP MOVSB

    POP DI
    POP SI
    POP CX
    POP ES
    POP BX
    RET
  PEGA_PINTURA ENDP

;****************************************************************************
; Almacena los valores.
;****************************************************************************

  VIEJO_MODO DB 0

CAMINO ENDS

END