;****************************************************************************
; MODULO PARA MODO DE VIDEO 19
;
; PRUEBA 7 - Pega un cuadro en la pantalla.
;
; Recuerde llamar MMV19_TOMA_SEGMENTO para establecer el segmento de la
; pantalla.
;****************************************************************************

;****************************************************************************
; Define los valores.
;****************************************************************************

  PUBLIC MMV19_INICIA_MODO
  PUBLIC MMV19_FINALIZA_MODO
  PUBLIC MMV19_TOMA_SEGMENTO
  PUBLIC MMV19_TRANSFORMA_X_Y
  PUBLIC MMV19_PEGA_PUNTO
  PUBLIC MMV19_PEGA_LINEA_HORIZONTAL
  PUBLIC MMV19_PEGA_LINEA_VERTICAL
  PUBLIC MMV19_PEGA_PINTURA
  PUBLIC MMV19_PEGA_CUADRO

  INTERRUPCION_VIDEO = 010H
  FUNCION_PONE_MODO  = 000H
  MODO               = 19

  SEGMENTO_BIOS = 00040H
  CAMPO_MODO    = 00049H

  SEGMENTO_VIDEO = 0A000H

  ANCHO_PANTALLA = 320

CAMINO SEGMENT PUBLIC
  ASSUME CS:CAMINO,DS:CAMINO,SS:CAMINO,ES:CAMINO

;****************************************************************************
; Guarda el modo de video y pone el 19. Este modo tiene un rea de 320 * 200,
; y 256 colores.
;
; Escribe:
;   VIEJO_MODO
;****************************************************************************

  LABEL MMV19_INICIA_MODO NEAR
  INICIA_MODO PROC NEAR
    PUSH BX
    PUSH ES
    PUSH AX

;****************************************************************************
; Obtiene el modo de video. Este valor se encuentra en el segmento de datos
; del BIOS.
    MOV BX,SEGMENTO_BIOS
    MOV ES,BX
    MOV BX,CAMPO_MODO
    MOV AL,ES:[BX]

;****************************************************************************
; Almacena su valor.
    MOV VIEJO_MODO,AL

;****************************************************************************
; Cambia al nuevo modo.
    MOV AH,FUNCION_PONE_MODO
    MOV AL,MODO
    INT INTERRUPCION_VIDEO

    POP AX
    POP ES
    POP BX
    RET
  INICIA_MODO ENDP

;****************************************************************************
; Pone el viejo modo de video.
;
; Lee:
;   VIEJO_MODO
;****************************************************************************

  LABEL MMV19_FINALIZA_MODO NEAR
  FINALIZA_MODO PROC NEAR
    PUSH AX

    MOV AH,FUNCION_PONE_MODO
    MOV AL,VIEJO_MODO
    INT INTERRUPCION_VIDEO

    POP AX
    RET
  FINALIZA_MODO ENDP

;****************************************************************************
; Toma el segmento de video.
;
; Regresa:
;   ES = segmento de video
;      = 0A000H
;****************************************************************************

  LABEL MMV19_TOMA_SEGMENTO NEAR
  TOMA_SEGMENTO PROC NEAR
    PUSH AX

    MOV AX,SEGMENTO_VIDEO
    MOV ES,AX

    POP AX
    RET
  TOMA_SEGMENTO ENDP

;****************************************************************************
; Transforma las coordenadas (X,Y) en una posicin en la pantalla. La
; frmula es:
;   P = X + Y * ANCHO.
;
; La versin mejorada para el 80x86 es:
;   P = X + Y * 256 + Y * 64 = X + Y SHL 8 + Y SHL 6
;
; Necesita:
;   AX = X
;   BX = Y
;
; Regresa:
;   DI = posicin en la pantalla
;****************************************************************************

  LABEL MMV19_TRANSFORMA_X_Y NEAR
  TRANSFORMA_X_Y PROC NEAR
    PUSH AX
    PUSH BX

;    MOV DI,AX
;    MOV AX,BX
;    MOV BX,ANCHO_PANTALLA
;    MUL BX
;    ADD DI,AX

;****************************************************************************
; Asigna X al resultado.
    MOV DI,AX

;****************************************************************************
; Multiplica Y por 256 y suma al resultado.
    XCHG BL,BH
    ADD DI,BX

;****************************************************************************
; Multiplica Y por 64 y suma al resultado.
    SHR BX,2
    ADD DI,BX

    POP BX
    POP AX
    RET
  TRANSFORMA_X_Y ENDP

;****************************************************************************
; Pega un punto en la pantalla.
;
; Necesita:
;   AL    = color del punto
;         = 0..255
;         = 0, negro en la paleta convencional
;         = 15, blanco en la paleta convencional
;   ES:DI = posicin en la pantalla
;         = 0..63999
;         = 0, la esquina superior izquierda
;         = 63999, la esquina inferior derecha
;
; Escribe:
;   ES:DI
;****************************************************************************

  LABEL MMV19_PEGA_PUNTO NEAR
  PONE_PUNTO PROC NEAR
    PUSH DI

    STOSB

    POP DI
    RET
  PONE_PUNTO ENDP

;****************************************************************************
; Pega una lnea horizontal en la pantalla.
;
; Necesita:
;   AL    = color del punto
;   ES:DI = inicio de la lnea (posicin superior izquierda)
;   CX    = cantidad de puntos en la lnea
;         = 1..64000
;
; Escribe:
;   ES:DI
;****************************************************************************

  LABEL MMV19_PEGA_LINEA_HORIZONTAL NEAR
  PEGA_LINEA_HORIZONTAL PROC NEAR
    PUSH DI
    PUSH CX

    REP STOSB

    POP CX
    POP DI
    RET
  PEGA_LINEA_HORIZONTAL ENDP

;****************************************************************************
; Pega una lnea vertical en la pantalla.
;
; Necesita:
;   AL    = color del punto
;   ES:DI = inicio de la lnea (posicin superior)
;   CX    = cantidad de puntos en la lnea
;
; Escribe:
;   ES:DI
;****************************************************************************

  LABEL MMV19_PEGA_LINEA_VERTICAL NEAR
  PEGA_LINEA_VERTICAL PROC NEAR
    PUSH DI
    PUSH CX

    CICLO_1:

;****************************************************************************
; Pega un punto y salta a la siguiente lnea en la pantalla.
    STOSB
    ADD DI,ANCHO_PANTALLA-1
    LOOP CICLO_1

    POP CX
    POP DI
    RET
  PEGA_LINEA_VERTICAL ENDP

;****************************************************************************
; Pega una pintura en la pantalla.
;
; Necesita:
;   DS:SI = inicio de la pintura
;   ES:DI = inicio en pantalla (posicin superior)
;   CX    = cantidad de puntos a pegar
;
; Lee:
;   DS:SI
;
; Escribe:
;   ES:DI
;****************************************************************************

  LABEL MMV19_PEGA_PINTURA NEAR
  PEGA_PINTURA PROC NEAR
    PUSH BX
    PUSH CX
    PUSH SI
    PUSH DI

;****************************************************************************
; Calcula la cantidad de words y bytes a pegar.
    MOV BX,CX
    AND BX,1
    SHR CX,1

;****************************************************************************
; Pega los words.
    REP MOVSW

;****************************************************************************
; Pega los bytes restantes.
    MOV CX,BX
    REP MOVSB

    POP DI
    POP SI
    POP CX
    POP BX
    RET
  PEGA_PINTURA ENDP

;****************************************************************************
; Pega un cuadro en la pantalla.
;
; Necesita:
;   DS:SI = inicio de la pintura
;   CX    = ancho del cuadro
;   DX    = alto del cuadro
;   ES:DI = inicio en pantalla (posicin superior)
;
; Llama:
;   PEGA_PINTURA
;****************************************************************************

  LABEL MMV19_PEGA_CUADRO NEAR
  PEGA_CUADRO PROC NEAR
    PUSH CX
    PUSH DX
    PUSH SI
    PUSH DI

;****************************************************************************
; Prepara CX para los cambios que sufrir.
    XCHG CX,DX

    CICLO_2:

;****************************************************************************
; Convierte CX en el ancho del cuadro.
    XCHG CX,DX

    CALL PEGA_PINTURA

    ADD SI,CX

;****************************************************************************
; Convierte CX en el contador del ciclo.
    XCHG CX,DX

;****************************************************************************
; Pasa a la siguiente lnea.
    ADD DI,ANCHO_PANTALLA

    LOOP CICLO_2

    POP DI
    POP SI
    POP DX
    POP CX
    RET
  PEGA_CUADRO ENDP

;****************************************************************************
; Almacena los valores.
;****************************************************************************

  VIEJO_MODO DB 0

CAMINO ENDS

END