;****************************************************************************
; MODULO PARA MODO DE VIDEO 19
;
; PRUEBA 12 - Espera el regreso del caon de la pantalla.
;
; Recuerde llamar MMV19_TOMA_SEGMENTO para establecer el segmento de la
; pantalla.
;****************************************************************************

;****************************************************************************
; Define los valores.
;****************************************************************************

  PUBLIC MMV19_INICIA_MODO
  PUBLIC MMV19_FINALIZA_MODO
  PUBLIC MMV19_TOMA_SEGMENTO
  PUBLIC MMV19_TRANSFORMA_X_Y

  PUBLIC MMV19_PEGA_PUNTO
  PUBLIC MMV19_COPIA_PUNTO

  PUBLIC MMV19_LLENA_PINTURA
  PUBLIC MMV19_PEGA_PINTURA
  PUBLIC MMV19_COPIA_PINTURA
  PUBLIC MMV19_PEGA_PINTURA_HUECO
  PUBLIC MMV19_COPIA_PINTURA_HUECO

  PUBLIC MMV19_LLENA_CUADRO
  PUBLIC MMV19_PEGA_CUADRO
  PUBLIC MMV19_COPIA_CUADRO
  PUBLIC MMV19_PEGA_CUADRO_HUECO
  PUBLIC MMV19_COPIA_CUADRO_HUECO

  PUBLIC MMV19_LLENA_MEZCLA
  PUBLIC MMV19_PEGA_MEZCLA
  PUBLIC MMV19_COPIA_MEZCLA

  PUBLIC MMV19_ESPERA_ACTUALIZACION

  INTERRUPCION_VIDEO = 010H
  FUNCION_PONE_MODO  = 000H
  MODO               = 19

  SEGMENTO_BIOS = 00040H
  CAMPO_MODO    = 00049H

  SEGMENTO_VIDEO = 0A000H

  ANCHO_PANTALLA = 320

  PUERTO_LECTURA     = 03C7H
  PUERTO_ESCRITURA   = 03C8H
  PUERTO_TRANSMISION = 03C9H

  PUERTO_ACTUALIZACION = 03DAH

CAMINO SEGMENT PUBLIC
  ASSUME CS:CAMINO,DS:CAMINO,SS:CAMINO,ES:CAMINO

;****************************************************************************
; Guarda el modo de video y pone el 19. Este modo tiene un rea de 320 * 200,
; y 256 colores.
;
; Escribe:
;   VIEJO_MODO
;****************************************************************************

  LABEL MMV19_INICIA_MODO NEAR
  INICIA_MODO PROC NEAR
    PUSH BX
    PUSH ES
    PUSH AX

;----------------------------------------------------------------------------
; Obtiene el modo de video. Este valor se encuentra en el segmento de datos
; del BIOS.
    MOV BX,SEGMENTO_BIOS
    MOV ES,BX
    MOV BX,CAMPO_MODO
    MOV AL,ES:[BX]

;----------------------------------------------------------------------------
; Almacena su valor.
    MOV VIEJO_MODO,AL

;----------------------------------------------------------------------------
; Cambia al nuevo modo.
    MOV AH,FUNCION_PONE_MODO
    MOV AL,MODO
    INT INTERRUPCION_VIDEO

    POP AX
    POP ES
    POP BX
    RET
  INICIA_MODO ENDP

;****************************************************************************
; Pone el viejo modo de video.
;
; Lee:
;   VIEJO_MODO
;****************************************************************************

  LABEL MMV19_FINALIZA_MODO NEAR
  FINALIZA_MODO PROC NEAR
    PUSH AX

    MOV AH,FUNCION_PONE_MODO
    MOV AL,VIEJO_MODO
    INT INTERRUPCION_VIDEO

    POP AX
    RET
  FINALIZA_MODO ENDP

;****************************************************************************
; Toma el segmento de video.
;
; Regresa:
;   ES = segmento de video
;      = 0A000H
;****************************************************************************

  LABEL MMV19_TOMA_SEGMENTO NEAR
  TOMA_SEGMENTO PROC NEAR
    PUSH AX

    MOV AX,SEGMENTO_VIDEO
    MOV ES,AX

    POP AX
    RET
  TOMA_SEGMENTO ENDP

;****************************************************************************
; Transforma las coordenadas (X,Y) en una posicin en la pantalla. La
; frmula es:
;   P = X + Y * ANCHO.
;
; La versin mejorada para el 80x86 es:
;   P = X + Y * 256 + Y * 64 = X + Y SHL 8 + Y SHL 6
;
; Necesita:
;   AX = X
;   BX = Y
;
; Regresa:
;   DI = posicin en la pantalla
;****************************************************************************

  LABEL MMV19_TRANSFORMA_X_Y NEAR
  TRANSFORMA_X_Y PROC NEAR
    PUSH AX
    PUSH BX

;    MOV DI,AX
;    MOV AX,BX
;    MOV BX,ANCHO_PANTALLA
;    MUL BX
;    ADD DI,AX

;----------------------------------------------------------------------------
; Asigna X al resultado.
    MOV DI,AX

;----------------------------------------------------------------------------
; Multiplica Y por 256 y suma al resultado.
    XCHG BL,BH
    ADD DI,BX

;----------------------------------------------------------------------------
; Multiplica Y por 64 y suma al resultado.
    SHR BX,2
    ADD DI,BX

    POP BX
    POP AX
    RET
  TRANSFORMA_X_Y ENDP

;****************************************************************************
; Pega un punto en la pantalla.
;
; Necesita:
;   AL    = color del punto
;   ES:DI = posicin en la pantalla (DI = 0..63999)
;
; Escribe:
;   ES:DI
;****************************************************************************

  LABEL MMV19_PEGA_PUNTO NEAR
  PONE_PUNTO PROC NEAR
    PUSH DI

    STOSB

    POP DI
    RET
  PONE_PUNTO ENDP

;****************************************************************************
; Copia un punto de la pantalla.
;
; Necesita:
;   DS:SI = posicin en la pantalla
;
; Regresa:
;   AL = color del punto
;
; Lee:
;   DS:SI
;****************************************************************************

  LABEL MMV19_COPIA_PUNTO NEAR
  COPIA_PUNTO PROC NEAR
    PUSH SI

    LODSB

    POP SI
    RET
  COPIA_PUNTO ENDP

;****************************************************************************
; Pega una lnea de puntos en la pantalla.
;
; Necesita:
;   AL    = color del punto
;   ES:DI = inicio de la lnea (posicin superior izquierda)
;   CX    = cantidad de puntos en la lnea
;         = 1..64000
;
; Escribe:
;   ES:DI
;****************************************************************************

  LABEL MMV19_LLENA_PINTURA NEAR
  LLENA_PINTURA PROC NEAR
    PUSH BX
    PUSH CX
    PUSH DI

;----------------------------------------------------------------------------
; Calcula la cantidad de words y bytes a pegar.
    MOV BX,CX
    AND BX,1
    SHR CX,1

;----------------------------------------------------------------------------
; Pega los words.
    REP STOSW

;----------------------------------------------------------------------------
; Pega los bytes restantes.
    MOV CX,BX
    REP STOSB

    POP DI
    POP CX
    POP BX
    RET
  LLENA_PINTURA ENDP

;****************************************************************************
; Pega un conjunto de puntos en la pantalla.
;
; Necesita:
;   DS:SI = inicio de la pintura
;   ES:DI = inicio en pantalla (posicin superior)
;   CX    = cantidad de puntos a pegar
;
; Lee:
;   DS:SI
;
; Escribe:
;   ES:DI
;****************************************************************************

  LABEL MMV19_PEGA_PINTURA NEAR
  PEGA_PINTURA PROC NEAR
    PUSH BX
    PUSH CX
    PUSH SI
    PUSH DI

;----------------------------------------------------------------------------
; Calcula la cantidad de words y bytes a pegar.
    MOV BX,CX
    AND BX,1
    SHR CX,1

;----------------------------------------------------------------------------
; Pega los words.
    REP MOVSW

;----------------------------------------------------------------------------
; Pega los bytes restantes.
    MOV CX,BX
    REP MOVSB

    POP DI
    POP SI
    POP CX
    POP BX
    RET
  PEGA_PINTURA ENDP

;****************************************************************************
; Copia un conjunto de puntos de la pantalla.
;
; Necesita:
;   DS:SI = inicio en pantalla (posicin superior)
;   ES:DI = inicio de la pintura
;   CX    = cantidad de puntos a copiar
;
; Lee:
;   DS:SI
;
; Escribe:
;   ES:DI
;****************************************************************************

  LABEL MMV19_COPIA_PINTURA NEAR
  COPIA_PINTURA PROC NEAR
    PUSH BX
    PUSH CX
    PUSH SI
    PUSH DI

;----------------------------------------------------------------------------
; Calcula la cantidad de words y bytes a copiar.
    MOV BX,CX
    AND BX,1
    SHR CX,1

;----------------------------------------------------------------------------
; Copia los words.
    REP MOVSW

;----------------------------------------------------------------------------
; Copia los bytes restantes.
    MOV CX,BX
    REP MOVSB

    POP DI
    POP SI
    POP CX
    POP BX
    RET
  COPIA_PINTURA ENDP

;****************************************************************************
; Pega un conjunto de puntos en la pantalla excluyendo el 0.
;
; Necesita:
;   DS:SI = inicio de la pintura
;   ES:DI = inicio en pantalla (posicin superior)
;   CX    = cantidad de puntos a pegar
;
; Lee:
;   DS:SI
;
; Escribe:
;   ES:DI
;****************************************************************************

  LABEL MMV19_PEGA_PINTURA_HUECO NEAR
  PEGA_PINTURA_HUECO PROC NEAR
    PUSH BX
    PUSH CX
    PUSH SI
    PUSH DI

  PPH_REPITE:

;----------------------------------------------------------------------------
; Carga un byte de la fuente y salta si es 0.
    LODSB
    TEST AL,AL
    JZ PPH_CONTINUA

    STOSB
    LOOP PPH_REPITE
    JMP PPH_TERMINA

  PPH_CONTINUA:

;----------------------------------------------------------------------------
; Pasa a la siguiente posicin en la pantalla.
    INC DI

    LOOP PPH_REPITE

  PPH_TERMINA:

    POP DI
    POP SI
    POP CX
    POP BX
    RET
  PEGA_PINTURA_HUECO ENDP

;****************************************************************************
; Copia un conjunto de puntos de la pantalla excluyendo el 0.
;
; Necesita:
;   DS:SI = inicio en pantalla (posicin superior)
;   ES:DI = inicio de la pintura
;   CX    = cantidad de puntos a copiar
;
; Lee:
;   DS:SI
;
; Escribe:
;   ES:DI
;****************************************************************************

  LABEL MMV19_COPIA_PINTURA_HUECO NEAR
  COPIA_PINTURA_HUECO PROC NEAR
    PUSH BX
    PUSH CX
    PUSH SI
    PUSH DI

  CPH_REPITE:

;----------------------------------------------------------------------------
; Carga un byte de la fuente y salta si es 0.
    LODSB
    TEST AL,AL
    JZ CPH_CONTINUA

    STOSB
    LOOP CPH_REPITE
    JMP CPH_TERMINA

  CPH_CONTINUA:

;----------------------------------------------------------------------------
; Pasa a la siguiente posicin en la pantalla.
    INC DI

    LOOP CPH_REPITE

  CPH_TERMINA:

    POP DI
    POP SI
    POP CX
    POP BX
    RET
  COPIA_PINTURA_HUECO ENDP

;****************************************************************************
; Llena un conjunto de lneas en la pantalla.
;
; Necesita:
;   AL    = color del punto
;   CX    = ancho del cuadro
;   DX    = alto del cuadro
;   ES:DI = inicio en pantalla (posicin superior)
;
; Llama:
;   LLENA_PINTURA
;****************************************************************************

  LABEL MMV19_LLENA_CUADRO NEAR
  LLENA_CUADRO PROC NEAR
    PUSH CX
    PUSH DX
    PUSH SI
    PUSH DI

;----------------------------------------------------------------------------
; Prepara CX para los cambios que sufrir.
    XCHG CX,DX

  LC_REPITE:

;----------------------------------------------------------------------------
; Convierte CX en el ancho del cuadro.
    XCHG CX,DX

    CALL LLENA_PINTURA

;----------------------------------------------------------------------------
; Pasa a la siguiente lnea.
    ADD SI,CX
    ADD DI,ANCHO_PANTALLA

;----------------------------------------------------------------------------
; Convierte CX en el contador del ciclo.
    XCHG CX,DX

    LOOP LC_REPITE

    POP DI
    POP SI
    POP DX
    POP CX
    RET
  LLENA_CUADRO ENDP

;****************************************************************************
; Pega un conjunto de lneas de pintura en la pantalla.
;
; Necesita:
;   DS:SI = inicio de la pintura
;   CX    = ancho del cuadro
;   DX    = alto del cuadro
;   ES:DI = inicio en pantalla (posicin superior)
;
; Llama:
;   PEGA_PINTURA
;****************************************************************************

  LABEL MMV19_PEGA_CUADRO NEAR
  PEGA_CUADRO PROC NEAR
    PUSH CX
    PUSH DX
    PUSH SI
    PUSH DI

;----------------------------------------------------------------------------
; Prepara CX para los cambios que sufrir.
    XCHG CX,DX

  PC_REPITE:

;----------------------------------------------------------------------------
; Convierte CX en el ancho del cuadro.
    XCHG CX,DX

    CALL PEGA_PINTURA

;----------------------------------------------------------------------------
; Pasa a la siguiente lnea.
    ADD SI,CX
    ADD DI,ANCHO_PANTALLA

;----------------------------------------------------------------------------
; Convierte CX en el contador del ciclo.
    XCHG CX,DX

    LOOP PC_REPITE

    POP DI
    POP SI
    POP DX
    POP CX
    RET
  PEGA_CUADRO ENDP

;****************************************************************************
; Copia un conjunto de lneas de pintura de la pantalla.
;
; Necesita:
;   DS:SI = inicio en la pantalla (posicin superior)
;   ES:DI = inicio del cuadro
;   CX    = ancho del cuadro
;   DX    = alto del cuadro
;
; Llama:
;   COPIA_PINTURA
;****************************************************************************

  LABEL MMV19_COPIA_CUADRO NEAR
  COPIA_CUADRO PROC NEAR
    PUSH CX
    PUSH DX
    PUSH SI
    PUSH DI

;----------------------------------------------------------------------------
; Prepara CX para los cambios que sufrir.
    XCHG CX,DX

  CC_REPITE:

;----------------------------------------------------------------------------
; Convierte CX en el ancho del cuadro.
    XCHG CX,DX

    CALL COPIA_PINTURA

;----------------------------------------------------------------------------
; Pasa a la siguiente lnea.
    ADD SI,ANCHO_PANTALLA
    ADD DI,CX

;----------------------------------------------------------------------------
; Convierte CX en el contador del ciclo.
    XCHG CX,DX

    LOOP CC_REPITE

    POP DI
    POP SI
    POP DX
    POP CX
    RET
  COPIA_CUADRO ENDP

;****************************************************************************
; Pega un cuadro con un hueco en la pantalla.
;
; Necesita:
;   DS:SI = inicio de la pintura
;   CX    = ancho del cuadro
;   DX    = alto del cuadro
;   ES:DI = inicio en pantalla (posicin superior)
;
; Llama:
;   PEGA_PINTURA_HUECO
;****************************************************************************

  LABEL MMV19_PEGA_CUADRO_HUECO NEAR
  PEGA_CUADRO_HUECO PROC NEAR
    PUSH CX
    PUSH DX
    PUSH SI
    PUSH DI

;----------------------------------------------------------------------------
; Prepara CX para los cambios que sufrir.
    XCHG CX,DX

  PCH_REPITE:

;----------------------------------------------------------------------------
; Convierte CX en el ancho del cuadro.
    XCHG CX,DX

    CALL PEGA_PINTURA_HUECO

;----------------------------------------------------------------------------
; Pasa a la siguiente lnea.
    ADD SI,CX
    ADD DI,ANCHO_PANTALLA

;----------------------------------------------------------------------------
; Convierte CX en el contador del ciclo.
    XCHG CX,DX

    LOOP PCH_REPITE

    POP DI
    POP SI
    POP DX
    POP CX
    RET
  PEGA_CUADRO_HUECO ENDP

;****************************************************************************
; Copia un cuadro con un hueco de la pantalla.
;
; Necesita:
;   DS:SI = inicio en pantalla (posicin superior)
;   CX    = ancho del cuadro
;   DX    = alto del cuadro
;   ES:DI = inicio de la pintura
;
; Llama:
;   COPIA_PINTURA_HUECO
;****************************************************************************

  LABEL MMV19_COPIA_CUADRO_HUECO NEAR
  COPIA_CUADRO_HUECO PROC NEAR
    PUSH CX
    PUSH DX
    PUSH SI
    PUSH DI

;----------------------------------------------------------------------------
; Prepara CX para los cambios que sufrir.
    XCHG CX,DX

  CCH_REPITE:

;----------------------------------------------------------------------------
; Convierte CX en el ancho del cuadro.
    XCHG CX,DX

    CALL COPIA_PINTURA_HUECO

;----------------------------------------------------------------------------
; Pasa a la siguiente lnea.
    ADD SI,ANCHO_PANTALLA
    ADD DI,CX

;----------------------------------------------------------------------------
; Convierte CX en el contador del ciclo.
    XCHG CX,DX

    LOOP CCH_REPITE

    POP DI
    POP SI
    POP DX
    POP CX
    RET
  COPIA_CUADRO_HUECO ENDP

;****************************************************************************
; Llena la paleta con una intensidad.
;
; Necesita:
;   BL = intensidad
;   AL = color inicial en la paleta
;   CX = cantidad de intensidades a pegar
;****************************************************************************

  LABEL MMV19_LLENA_MEZCLA NEAR
  LLENA_MEZCLA PROC NEAR
    PUSH DX
    PUSH AX
    PUSH CX

    MOV DX,PUERTO_ESCRITURA
    OUT DX,AL
    MOV DX,PUERTO_TRANSMISION
    MOV AL,BL

  LM_REPITE:

    OUT DX,AL
    LOOP LM_REPITE

    POP CX
    POP AX
    POP DX
    RET
  LLENA_MEZCLA ENDP

;****************************************************************************
; Pega un conjunto de intensidades en la paleta.
;
; Necesita:
;   DS:SI = mezcla de intensidades (roja, verde y azul)
;   AL    = color inicial en la paleta
;   CX    = cantidad de intensidades a pegar
;
; Lee:
;   DS:SI
;****************************************************************************

  LABEL MMV19_PEGA_MEZCLA NEAR
  PEGA_MEZCLA PROC NEAR
    PUSH DX
    PUSH SI
    PUSH CX

    MOV DX,PUERTO_ESCRITURA
    OUT DX,AL
    MOV DX,PUERTO_TRANSMISION

  PM_REPITE:

    LODSB
    OUT DX,AL
    LOOP PM_REPITE

    POP CX
    POP SI
    POP DX
    RET
  PEGA_MEZCLA ENDP

;****************************************************************************
; Copia un conjunto de intensidades de la paleta.
;
; Necesita:
;   AL    = color inicial en la paleta
;   CX    = cantidad de intensidades a copiar
;   ES:DI = mezcla de intensidades
;
; Escribe:
;   ES:DI
;****************************************************************************

  LABEL MMV19_COPIA_MEZCLA NEAR
  COPIA_MEZCLA PROC NEAR
    PUSH DX
    PUSH AX
    PUSH DI
    PUSH CX

    MOV DX,PUERTO_LECTURA
    OUT DX,AL
    MOV DX,PUERTO_TRANSMISION

  CM_REPITE:

    IN AL,DX
    STOSB
    LOOP CM_REPITE

    POP CX
    POP DI
    POP AX
    POP DX
    RET
  COPIA_MEZCLA ENDP

;****************************************************************************
; Espera el refrescamiento que hace el can del CRT a la pantalla.
;****************************************************************************

  LABEL MMV19_ESPERA_ACTUALIZACION NEAR
  ESPERA_ACTUALIZACION PROC NEAR
    PUSH DX
    PUSH AX

    MOV DX,PUERTO_ACTUALIZACION

  ER_ESPERA_SUBE:

    IN AL,DX
    AND AL,8
    JNZ ER_ESPERA_SUBE

  ER_ESPERA_BAJA:

    IN AL,DX
    AND AL,8
    JZ ER_ESPERA_BAJA

    POP AX
    POP DX
    RET
  ESPERA_ACTUALIZACION ENDP

;****************************************************************************
; Almacena los valores.
;****************************************************************************

  VIEJO_MODO DB 0

CAMINO ENDS

END