;****************************************************************************
; MODULO DE VIDEO - VIDEO GRAPHICS ARRAY
;
; Recuerde asignar a ES y a DS el valor 0A000H antes de llamar a los
; procedimientos que los requieren.
;****************************************************************************

;****************************************************************************
; Define los valores.
;****************************************************************************

  PUBLIC MVVGA_INICIA_MODO
  PUBLIC MVVGA_FINALIZA_MODO
  PUBLIC MVVGA_TRANSFORMA_X_Y

  PUBLIC MVVGA_COPIA_PUNTO
  PUBLIC MVVGA_PEGA_PUNTO

  PUBLIC MVVGA_COPIA_PINTURA
  PUBLIC MVVGA_LLENA_PINTURA
  PUBLIC MVVGA_PEGA_PINTURA
  PUBLIC MVVGA_AHUECA_PINTURA
  PUBLIC MVVGA_REDUCE_PINTURA
  PUBLIC MVVGA_AUMENTA_PINTURA

  PUBLIC MVVGA_COPIA_CUADRO
  PUBLIC MVVGA_LLENA_CUADRO
  PUBLIC MVVGA_PEGA_CUADRO
  PUBLIC MVVGA_AHUECA_CUADRO
  PUBLIC MVVGA_REDUCE_CUADRO
  PUBLIC MVVGA_AUMENTA_CUADRO

  PUBLIC MVVGA_COPIA_MEZCLA
  PUBLIC MVVGA_LLENA_MEZCLA
  PUBLIC MVVGA_PEGA_MEZCLA

  PUBLIC MVVGA_ESPERA_ACTUALIZACION

  PUBLIC MVVGA_AUMENTO_HORIZONTAL
  PUBLIC MVVGA_AUMENTO_VERTICAL

  INTERRUPCION_VIDEO = 010H
  FUNCION_PONE_MODO  = 000H
  MODO               = 19

  SEGMENTO_BIOS = 00040H
  CAMPO_MODO    = 00049H

  ANCHO_PANTALLA = 320

  PUERTO_LECTURA     = 03C7H
  PUERTO_ESCRITURA   = 03C8H
  PUERTO_TRANSMISION = 03C9H

  PUERTO_ACTUALIZACION = 03DAH

CAMINO SEGMENT PUBLIC
  ASSUME CS:CAMINO,DS:CAMINO,SS:CAMINO,ES:CAMINO

  DB 'MVVGA 1998 Marco Antonio Alvarado'

;****************************************************************************
; Inicia el modo de video 19. Guarda el modo de video anterior. El 19 tiene
; un rea de 320 * 200, y 256 colores.
;
; Llama:
;   INTERRUPCION_VIDEO
;
; Escribe:
;   VIEJO_MODO
;****************************************************************************

  LABEL MVVGA_INICIA_MODO NEAR
  INICIA_MODO PROC NEAR
    PUSH BX
    PUSH ES
    PUSH AX

;----------------------------------------------------------------------------
; Obtiene el modo de video. Este valor se encuentra en el segmento de datos
; del BIOS.
    MOV BX,SEGMENTO_BIOS
    MOV ES,BX
    MOV BX,CAMPO_MODO
    MOV AL,ES:[BX]

;----------------------------------------------------------------------------
; Almacena su valor.
    MOV VIEJO_MODO,AL

;----------------------------------------------------------------------------
; Cambia al nuevo modo.
    MOV AH,FUNCION_PONE_MODO
    MOV AL,MODO
    INT INTERRUPCION_VIDEO

    POP AX
    POP ES
    POP BX
    RET
  INICIA_MODO ENDP

;****************************************************************************
; Finaliza el modo de video 19. Restaura el modo anterior.
;
; Llama:
;   INTERRUPCION_VIDEO
;
; Lee:
;   VIEJO_MODO
;****************************************************************************

  LABEL MVVGA_FINALIZA_MODO NEAR
  FINALIZA_MODO PROC NEAR
    PUSH AX

    MOV AH,FUNCION_PONE_MODO
    MOV AL,VIEJO_MODO
    INT INTERRUPCION_VIDEO

    POP AX
    RET
  FINALIZA_MODO ENDP

;****************************************************************************
; Transforma las coordenadas X y Y en una posicin en la pantalla. La
; frmula es:
;   P = X + Y * ANCHO_PANTALLA.
;
; Necesita:
;   AX = X (0..319)
;   BX = Y (0..199)
;
; Regresa:
;   DI = posicin en la pantalla
;****************************************************************************

  LABEL MVVGA_TRANSFORMA_X_Y NEAR
  TRANSFORMA_X_Y PROC NEAR
    PUSH AX
    PUSH BX

;    MOV DI,AX
;    MOV AX,BX
;    MOV BX,ANCHO_PANTALLA
;    MUL BX
;    ADD DI,AX

;----------------------------------------------------------------------------
; Asigna X al resultado.
    MOV DI,AX

;----------------------------------------------------------------------------
; Multiplica Y por 256 y suma al resultado.
    XCHG BL,BH
    ADD DI,BX

;----------------------------------------------------------------------------
; Multiplica Y por 64 y suma al resultado.
    SHR BX,2
    ADD DI,BX

    POP BX
    POP AX
    RET
  TRANSFORMA_X_Y ENDP

;****************************************************************************
; Copia un punto de la pantalla.
;
; Necesita:
;   DS:SI = posicin en la pantalla
;
; Regresa:
;   AL = color del punto
;
; Lee:
;   DS:SI
;****************************************************************************

  LABEL MVVGA_COPIA_PUNTO NEAR
  COPIA_PUNTO PROC NEAR
    PUSH SI

    LODSB

    POP SI
    RET
  COPIA_PUNTO ENDP

;****************************************************************************
; Pega un punto en la pantalla.
;
; Necesita:
;   AL    = color del punto
;   ES:DI = posicin en la pantalla (ES = 0A000H, DI = 0..63999)
;
; Escribe:
;   ES:DI
;****************************************************************************

  LABEL MVVGA_PEGA_PUNTO NEAR
  PONE_PUNTO PROC NEAR
    PUSH DI

    STOSB

    POP DI
    RET
  PONE_PUNTO ENDP

;****************************************************************************
; Copia una pintura de la pantalla. Un grupo de puntos forma una pintura.
;
; Necesita:
;   DS:SI = inicio en pantalla
;   ES:DI = inicio de la pintura
;   CX    = cantidad de puntos a copiar (1..64000)
;
; Lee:
;   DS:SI
;
; Escribe:
;   ES:DI
;****************************************************************************

  LABEL MVVGA_COPIA_PINTURA NEAR
  COPIA_PINTURA PROC NEAR
    PUSH BX
    PUSH CX
    PUSH SI
    PUSH DI

;----------------------------------------------------------------------------
; Calcula la cantidad de words y bytes a copiar.
    MOV BX,CX
    AND BX,1
    SHR CX,1

;----------------------------------------------------------------------------
; Copia los words.
    REP MOVSW

;----------------------------------------------------------------------------
; Copia los bytes restantes.
    MOV CX,BX
    REP MOVSB

    POP DI
    POP SI
    POP CX
    POP BX
    RET
  COPIA_PINTURA ENDP

;****************************************************************************
; Llena una pintura en la pantalla con un mismo color.
;
; Necesita:
;   AL    = color del punto
;   ES:DI = inicio de la lnea
;   CX    = cantidad de puntos en la lnea
;
; Escribe:
;   ES:DI
;****************************************************************************

  LABEL MVVGA_LLENA_PINTURA NEAR
  LLENA_PINTURA PROC NEAR
    PUSH BX
    PUSH CX
    PUSH DI

;----------------------------------------------------------------------------
; Calcula la cantidad de words y bytes a pegar.
    MOV BX,CX
    AND BX,1
    SHR CX,1

;----------------------------------------------------------------------------
; Pega los words.
    REP STOSW

;----------------------------------------------------------------------------
; Pega los bytes restantes.
    MOV CX,BX
    REP STOSB

    POP DI
    POP CX
    POP BX
    RET
  LLENA_PINTURA ENDP

;****************************************************************************
; Pega una pintura en la pantalla.
;
; Necesita:
;   DS:SI = inicio de la pintura
;   ES:DI = inicio en pantalla
;   CX    = cantidad de puntos a pegar
;
; Lee:
;   DS:SI
;
; Escribe:
;   ES:DI
;****************************************************************************

  LABEL MVVGA_PEGA_PINTURA NEAR
  PEGA_PINTURA PROC NEAR
    PUSH BX
    PUSH CX
    PUSH SI
    PUSH DI

;----------------------------------------------------------------------------
; Calcula la cantidad de words y bytes a pegar.
    MOV BX,CX
    AND BX,1
    SHR CX,1

;----------------------------------------------------------------------------
; Pega los words.
    REP MOVSW

;----------------------------------------------------------------------------
; Pega los bytes restantes.
    MOV CX,BX
    REP MOVSB

    POP DI
    POP SI
    POP CX
    POP BX
    RET
  PEGA_PINTURA ENDP

;****************************************************************************
; Ahueca una pintura en la pantalla. Excluye los puntos de color 0, que es
; negro para la paleta convencional.
;
; Necesita:
;   DS:SI = inicio de la pintura
;   ES:DI = inicio en pantalla
;   CX    = cantidad de puntos a pegar
;
; Lee:
;   DS:SI
;
; Escribe:
;   ES:DI
;****************************************************************************

  LABEL MVVGA_AHUECA_PINTURA NEAR
  AHUECA_PINTURA PROC NEAR
    PUSH BX
    PUSH CX
    PUSH SI
    PUSH DI

  PPH_REPITE:

;----------------------------------------------------------------------------
; Continua si el byte de la pintura es 0.
    LODSB
    TEST AL,AL
    JZ PPH_CONTINUA

;----------------------------------------------------------------------------
; Pega el punto.
    STOSB
    LOOP PPH_REPITE
    JMP PPH_TERMINA

  PPH_CONTINUA:

;----------------------------------------------------------------------------
; Pasa a la siguiente posicin en la pantalla y repite.
    INC DI
    LOOP PPH_REPITE

  PPH_TERMINA:

    POP DI
    POP SI
    POP CX
    POP BX
    RET
  AHUECA_PINTURA ENDP

;****************************************************************************
; Reduce una pintura en la pantalla. Excluye los puntos que indica el mapa de
; transformacin.
;
; Necesita:
;   DS:SI = inicio de la pintura
;   ES:DI = inicio en pantalla
;   CX    = cantidad de puntos a pegar
;   AX    = mapa binario de transformacin
;             bit = 0, incluye
;                 = 1, excluye
;
; Lee:
;   DS:SI
;
; Escribe:
;   ES:DI
;****************************************************************************

  LABEL MVVGA_REDUCE_PINTURA NEAR
  REDUCE_PINTURA PROC NEAR
    PUSH AX
    PUSH SI
    PUSH DI
    PUSH CX

  RP_REPITE:

;----------------------------------------------------------------------------
; Salta si el siguiente bit del mapa horizontal es 1.
    ROR AX,1
    JC RP_CONTINUA

;----------------------------------------------------------------------------
; Pega el punto y repite.
    MOVSB
    LOOP RP_REPITE
    JMP RP_FINALIZA

  RP_CONTINUA:

;----------------------------------------------------------------------------
; Pasa al siguiente punto en la pintura y repite.
    INC SI
    LOOP RP_REPITE

  RP_FINALIZA:

    POP CX
    POP DI
    POP SI
    POP AX
    RET
  REDUCE_PINTURA ENDP

;****************************************************************************
; Aumenta una pintura en la pantalla. Repite los puntos que indica el mapa de
; transformacin.
;
; Necesita:
;   DS:SI = inicio de la pintura
;   ES:DI = inicio en pantalla
;   CX    = cantidad de puntos a pegar
;   AX    = mapa binario de transformacin
;             bit = 0, pega
;                   1, pega el punto otra vez
;
; Lee:
;   DS:SI
;   AUMENTO_HORIZONTAL
;
; Escribe:
;   ES:DI
;****************************************************************************

  LABEL MVVGA_AUMENTA_PINTURA NEAR
  AUMENTA_PINTURA PROC NEAR
    PUSH AX
    PUSH SI
    PUSH DI
    PUSH CX

  AP_REPITE:

;----------------------------------------------------------------------------
; Continua si el siguiente bit en el mapa horizontal es 1.
    ROR AX,1
    PUSH CX
    MOV CX,AUMENTO_HORIZONTAL
    JC AP_CONTINUA

;----------------------------------------------------------------------------
; Reduce el aumento horizontal en 1.
    DEC CX

  AP_CONTINUA:

;----------------------------------------------------------------------------
; Pega varias veces el punto de la pintura en la pantalla y repite.
    PUSH AX
    LODSB
    REP STOSB
    POP AX
    POP CX
    LOOP AP_REPITE

    POP CX
    POP DI
    POP SI
    POP AX
    RET
  AUMENTA_PINTURA ENDP

;****************************************************************************
; Copia un cuadro de la pantalla. Un grupo de lneas de pintura forman un
; cuadro. Las lneas se ubican en un bloque rectangular.
;
; Necesita:
;   DS:SI = inicio en la pantalla
;   ES:DI = inicio de la pintura
;   CX    = ancho del cuadro (1..320)
;   DX    = alto del cuadro (1..200)
;
; Llama:
;   COPIA_PINTURA
;****************************************************************************

  LABEL MVVGA_COPIA_CUADRO NEAR
  COPIA_CUADRO PROC NEAR
    PUSH CX
    PUSH DX
    PUSH SI
    PUSH DI

    XCHG CX,DX

  CC_REPITE:

;----------------------------------------------------------------------------
; Copia la lnea de pintura.
    XCHG CX,DX
    CALL COPIA_PINTURA

;----------------------------------------------------------------------------
; Pasa a la siguiente lnea y repite.
    ADD SI,ANCHO_PANTALLA
    ADD DI,CX
    XCHG CX,DX
    LOOP CC_REPITE

    POP DI
    POP SI
    POP DX
    POP CX
    RET
  COPIA_CUADRO ENDP

;****************************************************************************
; Llena un cuadro en la pantalla con lneas del mismo color.
;
; Necesita:
;   AL    = color del punto
;   ES:DI = inicio en pantalla
;   CX    = ancho del cuadro
;   DX    = alto del cuadro
;
; Llama:
;   LLENA_PINTURA
;****************************************************************************

  LABEL MVVGA_LLENA_CUADRO NEAR
  LLENA_CUADRO PROC NEAR
    PUSH CX
    PUSH DX
    PUSH SI
    PUSH DI

    XCHG CX,DX

  LC_REPITE:

;----------------------------------------------------------------------------
; Llena la lnea de pintura con el mismo color.
    XCHG CX,DX
    CALL LLENA_PINTURA

;----------------------------------------------------------------------------
; Pasa a la siguiente lnea y repite.
    ADD SI,CX
    ADD DI,ANCHO_PANTALLA
    XCHG CX,DX
    LOOP LC_REPITE

    POP DI
    POP SI
    POP DX
    POP CX
    RET
  LLENA_CUADRO ENDP

;****************************************************************************
; Pega un cuadro en la pantalla.
;
; Necesita:
;   DS:SI = inicio de la pintura
;   ES:DI = inicio en pantalla
;   CX    = ancho del cuadro
;   DX    = alto del cuadro
;
; Llama:
;   PEGA_PINTURA
;****************************************************************************

  LABEL MVVGA_PEGA_CUADRO NEAR
  PEGA_CUADRO PROC NEAR
    PUSH CX
    PUSH DX
    PUSH SI
    PUSH DI

    XCHG CX,DX

  PC_REPITE:

;----------------------------------------------------------------------------
; Pega la lnea de pintura en la pantalla.
    XCHG CX,DX
    CALL PEGA_PINTURA

;----------------------------------------------------------------------------
; Pasa a la siguiente lnea y repite.
    ADD SI,CX
    ADD DI,ANCHO_PANTALLA
    XCHG CX,DX
    LOOP PC_REPITE

    POP DI
    POP SI
    POP DX
    POP CX
    RET
  PEGA_CUADRO ENDP

;****************************************************************************
; Ahueca un cuadro en la pantalla.
;
; Necesita:
;   DS:SI = inicio de la pintura
;   ES:DI = inicio en pantalla
;   CX    = ancho del cuadro
;   DX    = alto del cuadro
;
; Lee:
;   DS:SI
;
; Escribe:
;   ES:DI
;
; Llama:
;   AHUECA_PINTURA
;****************************************************************************

  LABEL MVVGA_AHUECA_CUADRO NEAR
  AHUECA_CUADRO PROC NEAR
    PUSH CX
    PUSH DX
    PUSH SI
    PUSH DI

    XCHG CX,DX

  PCH_REPITE:

;----------------------------------------------------------------------------
; Ahueca la lnea de pintura.
    XCHG CX,DX
    CALL AHUECA_PINTURA

;----------------------------------------------------------------------------
; Pasa a la siguiente lnea y repite.
    ADD SI,CX
    ADD DI,ANCHO_PANTALLA
    XCHG CX,DX
    LOOP PCH_REPITE

    POP DI
    POP SI
    POP DX
    POP CX
    RET
  AHUECA_CUADRO ENDP

;****************************************************************************
; Reduce un cuadro en la pantalla. Excluye las lneas que indica el mapa de
; transformacin.
;
; Necesita:
;   DS:SI = inicio de la pintura
;   ES:DI = inicio en pantalla
;   CX    = ancho original del cuadro
;   DX    = alto original del cuadro
;   AX    = mapa binario de transformacin horizontal
;   BX    = mapa binario de transformacin vertical
;             bit = 0, incluye
;                 = 1, excluye
;
; Llama:
;   REDUCE_PINTURA
;****************************************************************************

  LABEL MVVGA_REDUCE_CUADRO NEAR
  REDUCE_CUADRO PROC NEAR
    PUSH BX
    PUSH CX
    PUSH DX
    PUSH SI
    PUSH DI

    XCHG CX,DX

  RC_REPITE:

;----------------------------------------------------------------------------
; Continua si el siguiente bit del mapa vertical es 1.
    ROR BX,1
    JC RC_CONTINUA

;----------------------------------------------------------------------------
; Reduce la lnea de pintura.
    XCHG CX,DX
    CALL REDUCE_PINTURA

;----------------------------------------------------------------------------
; Pasa a la siguiente lnea y repite.
    ADD SI,CX
    ADD DI,ANCHO_PANTALLA
    XCHG CX,DX
    LOOP RC_REPITE
    JMP RC_FINALIZA

  RC_CONTINUA:

;----------------------------------------------------------------------------
; Pasa a la siguiente lnea de la pintura y repite.
    ADD SI,CX
    XCHG CX,DX
    LOOP RC_REPITE

  RC_FINALIZA:

    POP DI
    POP SI
    POP DX
    POP CX
    POP BX
    RET
  REDUCE_CUADRO ENDP

;****************************************************************************
; Aumenta un cuadro en la pantalla. Repite las lneas que indica el mapa de
; transformacin.
;
; Necesita:
;   DS:SI = inicio de la pintura
;   ES:DI = inicio en pantalla
;   CX    = ancho original del cuadro
;   DX    = alto original del cuadro
;   AX    = mapa binario de transformacin horizontal
;   BX    = mapa binario de transformacin vertical
;             bit = 0, pega
;                 = 1, pega la lnea otra vez
;
; Lee:
;   DS:SI
;   AUMENTO_VERTICAL
;
; Escribe:
;   ES:DI
;
; Llama:
;   AUMENTA_PINTURA
;****************************************************************************

  LABEL MVVGA_AUMENTA_CUADRO NEAR
  AUMENTA_CUADRO PROC NEAR
    PUSH BX
    PUSH CX
    PUSH DX
    PUSH SI
    PUSH DI

    XCHG CX,DX

  AC_REPITE:

;----------------------------------------------------------------------------
; Continua si el siguiente bit del mapa vertical es 1.
    ROR BX,1
    PUSH BX
    MOV BX,AUMENTO_VERTICAL
    XCHG CX,DX
    XCHG BX,CX
    JC AC_CONTINUA

;----------------------------------------------------------------------------
; Reduce el aumento vertical en 1.
    DEC CX

  AC_CONTINUA:

    XCHG BX,CX
    CALL AUMENTA_PINTURA

;----------------------------------------------------------------------------
; Pasa a la siguiente lnea en la pantalla y repite.
    ADD DI,ANCHO_PANTALLA
    XCHG BX,CX
    LOOP AC_CONTINUA
    MOV CX,BX
    POP BX

;----------------------------------------------------------------------------
; Pasa a la siguiente lnea de la pintura y repite.
    ADD SI,CX
    XCHG CX,DX
    LOOP AC_REPITE

    POP DI
    POP SI
    POP DX
    POP CX
    POP BX
    RET
  AUMENTA_CUADRO ENDP

;****************************************************************************
; Copia una mezcla de la paleta. Un grupo de intensidades roja, verde y azul
; forman una mezcla que a la vez define un color.
;
; Necesita:
;   AL    = color inicial en la paleta
;   ES:DI = mezcla de intensidades (roja, verde y azul, con valores 0..63)
;   CX    = cantidad de intensidades a copiar
;
; Escribe:
;   ES:DI
;****************************************************************************

  LABEL MVVGA_COPIA_MEZCLA NEAR
  COPIA_MEZCLA PROC NEAR
    PUSH DX
    PUSH AX
    PUSH DI
    PUSH CX

    MOV DX,PUERTO_LECTURA
    OUT DX,AL
    MOV DX,PUERTO_TRANSMISION

  CM_REPITE:

    IN AL,DX
    STOSB
    LOOP CM_REPITE

    POP CX
    POP DI
    POP AX
    POP DX
    RET
  COPIA_MEZCLA ENDP

;****************************************************************************
; Llena la paleta con una intensidad.
;
; Necesita:
;   BL = intensidad (0..63)
;   AL = color inicial en la paleta
;   CX = cantidad de intensidades a pegar (1..256 * 3)
;****************************************************************************

  LABEL MVVGA_LLENA_MEZCLA NEAR
  LLENA_MEZCLA PROC NEAR
    PUSH DX
    PUSH AX
    PUSH CX

    MOV DX,PUERTO_ESCRITURA
    OUT DX,AL
    MOV DX,PUERTO_TRANSMISION
    MOV AL,BL

  LM_REPITE:

    OUT DX,AL
    LOOP LM_REPITE

    POP CX
    POP AX
    POP DX
    RET
  LLENA_MEZCLA ENDP

;****************************************************************************
; Pega una mezcla en la paleta.
;
; Necesita:
;   DS:SI = mezcla de intensidades
;   AL    = color inicial en la paleta
;   CX    = cantidad de intensidades a pegar
;
; Lee:
;   DS:SI
;****************************************************************************

  LABEL MVVGA_PEGA_MEZCLA NEAR
  PEGA_MEZCLA PROC NEAR
    PUSH DX
    PUSH SI
    PUSH CX

    MOV DX,PUERTO_ESCRITURA
    OUT DX,AL
    MOV DX,PUERTO_TRANSMISION

  PM_REPITE:

    LODSB
    OUT DX,AL
    LOOP PM_REPITE

    POP CX
    POP SI
    POP DX
    RET
  PEGA_MEZCLA ENDP

;****************************************************************************
; Espera la actualizacin de la pantalla. Aguarda que el caon del tubo de
; rayos catdicos (CRT) se desplace hasta la parte inferior de la pantalla.
; Justo despus de este paso se debe dibujar en la pantalla, con la finalidad
; que la imagen no parpade.
;****************************************************************************

  LABEL MVVGA_ESPERA_ACTUALIZACION NEAR
  ESPERA_ACTUALIZACION PROC NEAR
    PUSH DX
    PUSH AX

    MOV DX,PUERTO_ACTUALIZACION

  ER_ESPERA_SUBE:

    IN AL,DX
    AND AL,8
    JNZ ER_ESPERA_SUBE

  ER_ESPERA_BAJA:

    IN AL,DX
    AND AL,8
    JZ ER_ESPERA_BAJA

    POP AX
    POP DX
    RET
  ESPERA_ACTUALIZACION ENDP

;****************************************************************************
; Almacena los valores.
;****************************************************************************

  VIEJO_MODO DB 0

  LABEL MVVGA_AUMENTO_HORIZONTAL WORD
  AUMENTO_HORIZONTAL DW 2
  LABEL MVVGA_AUMENTO_VERTICAL WORD
  AUMENTO_VERTICAL   DW 2

CAMINO ENDS

END